==> tecfy_corev2_package-1.1.66.tar.gz: lib/views/components/entity/in_line_form.dart <==
part of tecfy_corev2_client_package;

class InLineForm extends StatefulWidget {
  final String? entityId;
  final Entity? entity;
  final Map<String, dynamic> data;
  final bool inPopup;
  final bool readOnly;
  final bool isAdd;
  final List<Property>? properties;
  final void Function(dynamic)? onDismiss;
  final void Function(dynamic)? onChanged;

  InLineForm(this.entityId, this.isAdd,
      {this.entity,
      required this.data,
      this.onChanged,
      this.onDismiss,
      this.properties,
      this.inPopup = false,
      this.readOnly = false});
  @override
  _InLineFormState createState() => _InLineFormState();
}

class _InLineFormState extends State<InLineForm> {
  late List<Property> _props;
  late List<PropertyNumber> _calcProps;
  late List<List<Property>> _lines;
  Entity? _entity;

  // Parser parser;
  @override
  void initState() {
    _entity =
        widget.entity ?? EntityService.getEntity(widget.entityId ?? '') ?? null;

    _props = widget.properties ??
        _entity?.properties
            .where((d) => widget.isAdd ? d.isAdd : d.isEdit)
            .toList() ??
        [];
    _lines = [[]];
    for (var p in _props) {
      _lines.last.add(p);
      if (p.newLineAfter) _lines.add([]);
    }
    _calcProps = _props
        .where((p) =>
            p.propertyType == PropertyTypes.Number &&
            (p as PropertyNumber).calcFormula != null &&
            p.calcFormula!.isNotEmpty)
        .toList()
        .cast<PropertyNumber>();
    super.initState();
  }

  _onUpdateData(Map<String, dynamic> data, Property property) {
    // we need this for Ramy's application to update the records when select an Item
    if (property.propertyType == PropertyTypes.Dropdown) {
      var prop = property as PropertyDropdown;
      if (MapService.getValue(data, prop.dbName) == null) return;
      var props = property.overwritePropertiesOnSelect?.keys ?? [];
      for (var key in props) {
        _onValueChanged(MapService.getValue(data, prop.dbName)[key],
            property.overwritePropertiesOnSelect![key]); //TODO: fix dots
      }
      setState(() {});
    }
  }

  Future _onValueChanging(value, propName) async {
    if (kDebugMode) print('inline value change! $propName');
    _entity?.updatedProps.clear();
    if (await _onValueChanged(value, propName)) {
      setState(() {});
    }
  }

  Future<bool> _onValueChanged(value, propName) async {
    if (kDebugMode) print('inline calc! $propName');
    if (_entity?.updatedProps[propName] != null) return false;
    if (_entity != null) _entity!.updatedProps[propName] = true;

    MapService.updateValue(widget.data, propName, value);

    bool updated = (await _entity?.setDefaultValues(widget.data)) ?? false;

    // for (Property p in props) {
    //   var r = await p.isVisible(widget.data, widget.entity);
    //   if (r != p.isVisable) {
    //     p.isVisable = r;
    updated = true;
    //   }
    // }

    // check the calced properties
    for (PropertyNumber p in _calcProps) {
      // if (p.calcedProps.contains(p.upperFirst(propName)) ||
      //     p.calcedProps.contains(propName)) {
      var vars = p.calcedProps
          .map((pro) =>
              MapService.getValue(widget.data, p.lowerFirst(pro).toString()))
          .toList();
      MapService.updateValue(widget.data, p.dbName, p.getCalcResult(vars));

      // }
    }

    // if (updated || _props.any((p) => p.filterValueField == propName))
    //   setState(() {});

    widget.onChanged?.call(widget.data);
    return updated;
  }

  @override
  Widget build(BuildContext context) {
    return widget.onDismiss == null || widget.readOnly
        ? _getData()
        : Stack(children: <Widget>[
            _getData(),
            Align(
                alignment: LocalConfigService.isArabic
                    ? Alignment.topLeft
                    : Alignment.topRight,
                child: IconButton(
                    icon: Icon(Icons.close),
                    onPressed: () => widget.onDismiss?.call(widget.data)))
          ]);
  }

  // get _getMargen => (widget.inPopup ?? false) ? 35 : 20;

  _getWidth(Property prop) {
    if (prop.fullWidth) return double.infinity;
    var fullwidth = LayoutService.getWidth(context);
    var isMobile = LayoutService.isMobile(context);
    if (!isMobile) fullwidth -= (ThemeService.menuWidth + 100);

    if (fullwidth > ThemeService.containerWidth) {
      fullwidth = ThemeService.containerWidth - 50;
    }
    if (fullwidth < 400) {
      return double.infinity; // means take full width
    }
    return fullwidth / 2 - (isMobile ? 20 : 0);
  }

  _getData() {
    return SizedBox(
      width: MediaQuery.of(context).size.width - ((widget.inPopup) ? 35 : 20),
      child: Card(
        color: ThemeService.backgroundSecondColor,
        margin: EdgeInsets.all(10),
        child: false
            ? Container(
                margin: EdgeInsets.all(0),
                constraints: BoxConstraints(maxWidth: double.infinity),
                padding: EdgeInsets.all(10),
                child: Column(
                  children: _lines
                      .map<Widget>(
                        (props) => Row(
                          children: List.generate(
                            props.length,
                            (i) => Expanded(
                              flex: props[i].size == PropertySizes.xSmall
                                  ? 1
                                  : props[i].size == PropertySizes.Small
                                      ? 2
                                      : props[i].size == PropertySizes.Default
                                          ? 3
                                          : props[i].size == PropertySizes.Large
                                              ? 4
                                              : props[i].size ==
                                                      PropertySizes.xLarge
                                                  ? 5
                                                  : 3,
                              child: PorpertyComponent(
                                readOnly: widget.readOnly,
                                property: props[i],
                                data: widget.data,
                                entity: _entity,
                                onChange: (v) =>
                                    _onValueChanging(v, props[i].dbName),
                                onUpdateData: (v) => _onUpdateData(v, props[i]),
                              ),
                            ),
                          ),
                        ),
                      )
                      .toList(),
                ),
              )
            : Wrap(
                children: List.generate(
                  _props.length,
                  (i) => Container(
                    margin: EdgeInsets.all(0),
                    constraints: BoxConstraints(maxWidth: _getWidth(_props[i])),
                    padding: EdgeInsets.all(10),
                    child: PorpertyComponent(
                      readOnly: widget.readOnly,
                      property: _props[i],
                      data: widget.data,
                      entity: _entity,
                      onChange: (v) => _onValueChanging(v, _props[i].dbName),
                      onUpdateData: (v) => _onUpdateData(v, _props[i]),
                    ),
                  ),
                ),
              ),
      ),
    );
  }
}

==> tecfy_corev2_package-1.1.84.tar.gz: lib/views/components/entity/in_line_form.dart <==
part of tecfy_corev2_client_package;

class InLineForm extends StatefulWidget {
  final String? entityId;
  final Entity? entity;
  final Map<String, dynamic> data;
  final bool inPopup;
  final bool readOnly;
  final bool isAdd;
  final List<Property>? properties;
  final void Function(dynamic)? onDismiss;
  final void Function(dynamic)? onChanged;

  InLineForm(this.entityId, this.isAdd,
      {this.entity,
      required this.data,
      this.onChanged,
      this.onDismiss,
      this.properties,
      this.inPopup = false,
      Key? key,
      this.readOnly = false})
      : super(key: key);
  @override
  _InLineFormState createState() => _InLineFormState();
}

class _InLineFormState extends State<InLineForm> {
  late List<Property> _props;
  late List<PropertyNumber> _calcProps;
  late List<List<Property>> _lines;
  Entity? _entity;

  // Parser parser;
  @override
  void initState() {
    _entity =
        widget.entity ?? EntityService.getEntity(widget.entityId ?? '') ?? null;

    _props = widget.properties ??
        _entity?.properties
            .where((d) => widget.isAdd ? d.isAdd : d.isEdit)
            .toList() ??
        [];
    _lines = [[]];
    for (var p in _props) {
      _lines.last.add(p);
      if (p.newLineAfter) _lines.add([]);
    }
    _calcProps = _props
        .where((p) =>
            p.propertyType == PropertyTypes.Number &&
            (p as PropertyNumber).calcFormula != null &&
            p.calcFormula!.isNotEmpty)
        .toList()
        .cast<PropertyNumber>();
    super.initState();
  }

  _onUpdateData(Map<String, dynamic> data, Property property) {
    // we need this for Ramy's application to update the records when select an Item
    if (property.propertyType == PropertyTypes.Dropdown) {
      var prop = property as PropertyDropdown;
      if (MapService.getValue(data, prop.dbName) == null) return;
      var props = property.overwritePropertiesOnSelect?.keys ?? [];
      for (var key in props) {
        _onValueChanged(MapService.getValue(data, prop.dbName)[key],
            property.overwritePropertiesOnSelect![key]); //TODO: fix dots
      }
      setState(() {});
    }
  }

  Future _onValueChanging(value, propName) async {
    if (kDebugMode) print('inline value change! $propName');
    _entity?.updatedProps.clear();
    if (await _onValueChanged(value, propName)) {
      setState(() {});
    }
  }

  Future<bool> _onValueChanged(value, propName) async {
    if (kDebugMode) print('inline calc! $propName');
    if (_entity?.updatedProps[propName] != null) return false;
    if (_entity != null) _entity!.updatedProps[propName] = true;

    MapService.updateValue(widget.data, propName, value);

    bool updated = (await _entity?.setDefaultValues(widget.data)) ?? false;

    // for (Property p in props) {
    //   var r = await p.isVisible(widget.data, widget.entity);
    //   if (r != p.isVisable) {
    //     p.isVisable = r;
    updated = true;
    //   }
    // }

    // check the calced properties
    for (PropertyNumber p in _calcProps) {
      // if (p.calcedProps.contains(p.upperFirst(propName)) ||
      //     p.calcedProps.contains(propName)) {
      var vars = p.calcedProps
          .map((pro) =>
              MapService.getValue(widget.data, p.lowerFirst(pro).toString()))
          .toList();
      MapService.updateValue(widget.data, p.dbName, p.getCalcResult(vars));

      // }
    }

    // if (updated || _props.any((p) => p.filterValueField == propName))
    //   setState(() {});

    widget.onChanged?.call(widget.data);
    return updated;
  }

  @override
  Widget build(BuildContext context) {
    return widget.onDismiss == null || widget.readOnly
        ? _getData()
        : Stack(children: <Widget>[
            _getData(),
            Align(
                alignment: LocalConfigService.isArabic
                    ? Alignment.topLeft
                    : Alignment.topRight,
                child: IconButton(
                    icon: Icon(Icons.close),
                    onPressed: () => widget.onDismiss?.call(widget.data)))
          ]);
  }

  // get _getMargen => (widget.inPopup ?? false) ? 35 : 20;

  _getWidth(Property prop) {
    if (prop.fullWidth) return double.infinity;
    var fullwidth = LayoutService.getWidth(context);
    var isMobile = LayoutService.isMobile(context);
    if (!isMobile) fullwidth -= (ThemeService.menuWidth + 100);

    if (fullwidth > ThemeService.containerWidth) {
      fullwidth = ThemeService.containerWidth - 50;
    }
    if (fullwidth < 400) {
      return double.infinity; // means take full width
    }
    return fullwidth / 2 - (isMobile ? 20 : 0);
  }

  _getData() {
    return SizedBox(
      width: MediaQuery.of(context).size.width - ((widget.inPopup) ? 35 : 20),
      child: Card(
        color: ThemeService.backgroundSecondColor,
        margin: EdgeInsets.all(10),
        child: false
            ? Container(
                margin: EdgeInsets.all(0),
                constraints: BoxConstraints(maxWidth: double.infinity),
                padding: EdgeInsets.all(10),
                child: Column(
                  children: _lines
                      .map<Widget>(
                        (props) => Row(
                          children: List.generate(
                            props.length,
                            (i) => Expanded(
                              flex: props[i].size == PropertySizes.xSmall
                                  ? 1
                                  : props[i].size == PropertySizes.Small
                                      ? 2
                                      : props[i].size == PropertySizes.Default
                                          ? 3
                                          : props[i].size == PropertySizes.Large
                                              ? 4
                                              : props[i].size ==
                                                      PropertySizes.xLarge
                                                  ? 5
                                                  : 3,
                              child: PorpertyComponent(
                                readOnly: widget.readOnly,
                                property: props[i],
                                data: widget.data,
                                entity: _entity,
                                onChange: (v) =>
                                    _onValueChanging(v, props[i].dbName),
                                onUpdateData: (v) => _onUpdateData(v, props[i]),
                              ),
                            ),
                          ),
                        ),
                      )
                      .toList(),
                ),
              )
            : Wrap(
                children: List.generate(
                  _props.length,
                  (i) => Container(
                    margin: EdgeInsets.all(0),
                    constraints: BoxConstraints(maxWidth: _getWidth(_props[i])),
                    padding: EdgeInsets.all(10),
                    child: PorpertyComponent(
                      readOnly: widget.readOnly,
                      property: _props[i],
                      data: widget.data,
                      entity: _entity,
                      onChange: (v) => _onValueChanging(v, _props[i].dbName),
                      onUpdateData: (v) => _onUpdateData(v, _props[i]),
                    ),
                  ),
                ),
              ),
      ),
    );
  }
}
